'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var Highlight = require('prism-react-renderer');
var PropTypes = require('prop-types');
var React = require('react');
var css = require('@emotion/css');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Highlight__default = /*#__PURE__*/_interopDefaultLegacy(Highlight);
var PropTypes__default = /*#__PURE__*/_interopDefaultLegacy(PropTypes);
var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

const ID = '(?:' +
    [
        /[a-zA-Z_\x80-\uFFFF][\w\x80-\uFFFF]*/.source,
        /-?(?:\.\d+|\d+(?:\.\d*)?)/.source,
        /"[^"\\]*(?:\\[\s\S][^"\\]*)*"/.source,
        /<(?:[^<>]|(?!<!--)<(?:[^<>"']|"[^"]*"|'[^']*')+>|<!--(?:[^-]|-(?!->))*-->)*>/.source,
    ].join('|') +
    ')';
const IDInside = {
    markup: {
        pattern: /(^<)[\s\S]+(?=>$)/,
        lookbehind: true,
        alias: ['language-markup', 'language-html', 'language-xml'],
        inside: Highlight.Prism.languages.markup,
    },
};
function withID(source, flags) {
    return RegExp(source.replace(/<ID>/g, function () {
        return ID;
    }), flags);
}
Highlight.Prism.languages['gv'] = Highlight.Prism.languages['dot'] = {
    comment: {
        pattern: /\/\/.*|\/\*[\s\S]*?\*\/|^#.*/m,
        greedy: true,
    },
    'graph-name': {
        pattern: withID(/(\b(?:digraph|graph|subgraph)[ \t\r\n]+)<ID>/.source, 'i'),
        lookbehind: true,
        greedy: true,
        alias: 'class-name',
        inside: IDInside,
    },
    'attr-value': {
        pattern: withID(/(=[ \t\r\n]*)<ID>/.source),
        lookbehind: true,
        greedy: true,
        inside: IDInside,
    },
    'attr-name': {
        pattern: withID(/([[;, \t\r\n])<ID>(?=[ \t\r\n]*=)/.source),
        lookbehind: true,
        greedy: true,
        inside: IDInside,
    },
    keyword: /\b(?:digraph|edge|graph|node|strict|subgraph)\b/i,
    'compass-point': {
        pattern: /(:[ \t\r\n]*)(?:[ns][ew]?|[ewc_])(?![\w\x80-\uFFFF])/,
        lookbehind: true,
        alias: 'builtin',
    },
    node: {
        pattern: withID(/(^|[^-.\w\x80-\uFFFF\\])<ID>/.source),
        lookbehind: true,
        greedy: true,
        inside: IDInside,
    },
    operator: /[=:]|-[->]/,
    punctuation: /[[\]{};,]/,
};

Highlight.Prism.languages['vb'] =
    Highlight.Prism.languages['vba'] =
        Highlight.Prism.languages['visual-basic'] =
            {
                comment: {
                    pattern: /(?:['‘’]|REM\b)(?:[^\r\n_]|_(?:\r\n?|\n)?)*/i,
                    inside: {
                        keyword: /^REM/i,
                    },
                },
                directive: {
                    pattern: /#(?:Const|Else|ElseIf|End|ExternalChecksum|ExternalSource|If|Region)(?:[^\S\r\n]_[^\S\r\n]*(?:\r\n?|\n)|.)+/i,
                    alias: 'comment',
                    greedy: true,
                },
                string: {
                    pattern: /\$?["“”](?:["“”]{2}|[^"“”])*["“”]C?/i,
                    greedy: true,
                },
                date: {
                    pattern: /#[^\S\r\n]*(?:\d+([/-])\d+\1\d+(?:[^\S\r\n]+(?:\d+[^\S\r\n]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[^\S\r\n]*(?:AM|PM))?))?|\d+[^\S\r\n]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[^\S\r\n]*(?:AM|PM))?)[^\S\r\n]*#/i,
                    alias: 'builtin',
                },
                number: /(?:(?:\b\d+(?:\.\d+)?|\.\d+)(?:E[+-]?\d+)?|&[HO][\dA-F]+)(?:U?[ILS]|[FRD])?/i,
                boolean: /\b(?:True|False|Nothing)\b/i,
                keyword: /\b(?:AddHandler|AddressOf|Alias|And(?:Also)?|As|Boolean|ByRef|Byte|ByVal|Call|Case|Catch|C(?:Bool|Byte|Char|Date|Dbl|Dec|Int|Lng|Obj|SByte|Short|Sng|Str|Type|UInt|ULng|UShort)|Char|Class|Const|Continue|Currency|Date|Decimal|Declare|Default|Delegate|Dim|DirectCast|Do|Double|Each|Else(?:If)?|End(?:If)?|Enum|Erase|Error|Event|Exit|Finally|For|Friend|Function|Get(?:Type|XMLNamespace)?|Global|GoSub|GoTo|Handles|If|Implements|Imports|In|Inherits|Integer|Interface|Is|IsNot|Let|Lib|Like|Long|Loop|Me|Mod|Module|Must(?:Inherit|Override)|My(?:Base|Class)|Namespace|Narrowing|New|Next|Not(?:Inheritable|Overridable)?|Object|Of|On|Operator|Option(?:al)?|Or(?:Else)?|Out|Overloads|Overridable|Overrides|ParamArray|Partial|Private|Property|Protected|Public|RaiseEvent|ReadOnly|ReDim|RemoveHandler|Resume|Return|SByte|Select|Set|Shadows|Shared|short|Single|Static|Step|Stop|String|Structure|Sub|SyncLock|Then|Throw|To|Try|TryCast|Type|TypeOf|U(?:Integer|Long|Short)|Using|Variant|Wend|When|While|Widening|With(?:Events)?|WriteOnly|Until|Xor)\b/i,
                operator: [
                    /[+\-*/\\^<=>&#@$%!]/,
                    {
                        pattern: /([^\S\r\n])_(?=[^\S\r\n]*[\r\n])/,
                        lookbehind: true,
                    },
                ],
                punctuation: /[{}().,:?]/,
            };

const vars = {
    border: '1px solid var(--yozora-colors-border-codeLineno, hsla(0deg, 0%, 80%, 0.8))',
    highlightBackground: 'var(--yozora-colors-background-codeHighlight, hsla(30deg, 90%, 50%, 0.3))',
};
const classes = {
    container: css.css({
        MozOsxFontSmoothing: 'grayscale',
        WebkitFontSmoothing: 'antialiased',
        display: 'flex',
        alignItems: 'stretch',
        overflow: 'hidden',
        width: '100%',
        padding: 0,
        transition: 'max-height 0.5s ease-in-out',
        tabSize: 2,
        fontSmooth: 'always',
        whiteSpace: 'pre',
        wordBreak: 'keep-all',
        wordSpacing: 'normal',
        wordWrap: 'normal',
    }),
    line: css.css({
        boxSizing: 'border-box',
        display: 'block',
        minWidth: 'fit-content',
        width: '100%',
        padding: '0 6px',
        letterSpacing: 'inherit',
        lineHeight: 'inherit',
        overflowWrap: 'inherit',
        tabSize: 'inherit',
        textIndent: 'inherit',
        textRendering: 'inherit',
        textTransform: 'inherit',
        whiteSpace: 'inherit',
        wordBreak: 'inherit',
        wordSpacing: 'inherit',
        wordWrap: 'inherit',
    }),
    highlightLine: css.css({
        background: vars.highlightBackground,
        borderColor: 'transparent',
    }),
    lineno: css.css({
        flex: '0 0 auto',
        overflow: 'hidden',
        boxSizing: 'border-box',
        padding: '0.5rem 0',
        cursor: 'default',
        userSelect: 'none',
        textAlign: 'right',
        borderRight: vars.border,
    }),
    codes: css.css({
        flex: '1 1 auto',
        overflow: 'overlay',
        boxSizing: 'border-box',
        padding: '0.5rem 0',
    }),
    codeWrapper: css.css({
        minWidth: '100%',
        width: 'fit-content',
    }),
    codeLine: css.css({
        padding: '0 1rem 0 12px',
    }),
};

const lineHeightRegex = /^([\d.]+)([\S]*)$/;
function calcHeight(lineHeight, countOfLines) {
    const value = Number(lineHeight);
    if (!Number.isNaN(value))
        return countOfLines * value;
    const m = lineHeightRegex.exec('' + lineHeight);
    if (m == null)
        return 0;
    const [, val, unit = 'px'] = m;
    return countOfLines * Number(val) + unit;
}

const CodeHighlighterContent = props => {
    const { codesRef, highlightLinenos = [], collapsed = false, maxLines = -1, lineHeight = '1.6rem', showLineNo = true, tokens, getLineProps, getTokenProps, onLinenoWidthChange, className, } = props;
    const countOfLines = maxLines > 0 ? maxLines : tokens.length;
    const linenoWidth = showLineNo
        ? `${Math.max(2, String(tokens.length).length) * 1.1}em`
        : undefined;
    const linenoRef = React__default["default"].useRef(null);
    const syncScrollEvents = React__default["default"].useCallback(e => {
        const codesArea = e.target;
        if (codesArea == null)
            return;
        const { scrollTop } = codesArea;
        if (linenoRef.current != null) {
            linenoRef.current.scrollTo(0, scrollTop);
        }
    }, []);
    const style = React__default["default"].useMemo(() => {
        const result = Object.assign(Object.assign({}, props.style), { lineHeight, maxHeight: 0, backgroundColor: 'none' });
        if (!collapsed) {
            const maxHeight = calcHeight(lineHeight, countOfLines + 1);
            result.maxHeight = maxHeight;
            result.minHeight = '100%';
        }
        return result;
    }, [props.style, lineHeight, countOfLines, collapsed]);
    React__default["default"].useEffect(() => {
        if (onLinenoWidthChange == null)
            return;
        onLinenoWidthChange(linenoWidth);
    }, [linenoWidth, onLinenoWidthChange]);
    return (React__default["default"].createElement("div", { className: css.cx('yozora-code-highlighter__content', classes.container, className), style: style },
        showLineNo && (React__default["default"].createElement("div", { key: "linenos", className: css.cx('yozora-code-highlighter__linenos', classes.lineno), style: { width: linenoWidth }, ref: linenoRef }, tokens.map((_, lineNo) => {
            const isHighlight = highlightLinenos.includes(lineNo + 1);
            return (React__default["default"].createElement("div", { key: lineNo, className: css.cx(classes.line, { [classes.highlightLine]: isHighlight }) },
                React__default["default"].createElement("span", { key: lineNo }, lineNo + 1)));
        }))),
        React__default["default"].createElement("div", { key: "codes", ref: codesRef, className: css.cx('yozora-code-highlighter__codes', classes.codes), onScroll: syncScrollEvents },
            React__default["default"].createElement("div", { className: classes.codeWrapper }, tokens.map((line, lineNo) => {
                const isHighlight = highlightLinenos.includes(lineNo + 1);
                const lineProps = getLineProps({ line });
                return (React__default["default"].createElement("div", Object.assign({}, lineProps, { key: lineNo, className: css.cx(classes.line, classes.codeLine, { [classes.highlightLine]: isHighlight }, lineProps.className) }), line.map((token, key) => (React__default["default"].createElement("span", Object.assign({}, getTokenProps({ token }), { key: key }))))));
            })))));
};
CodeHighlighterContent.propTypes = {
    codesRef: PropTypes__default["default"].any,
    highlightLinenos: PropTypes__default["default"].array,
    collapsed: PropTypes__default["default"].bool,
    maxLines: PropTypes__default["default"].number,
    lineHeight: PropTypes__default["default"].string,
    showLineNo: PropTypes__default["default"].bool,
    tokens: PropTypes__default["default"].array.isRequired,
    getLineProps: PropTypes__default["default"].func.isRequired,
    getTokenProps: PropTypes__default["default"].func.isRequired,
    onLinenoWidthChange: PropTypes__default["default"].func,
    className: PropTypes__default["default"].string,
    style: PropTypes__default["default"].any,
};

const vscDarkTheme = {
    plain: {
        color: '#d4d4d4',
        backgroundColor: '#1e1e1e',
    },
    styles: [
        {
            types: ['prolog'],
            style: { color: 'rgb(0, 0, 128)' },
        },
        {
            types: ['comment', 'punctuation'],
            style: { color: 'rgb(106, 153, 85)' },
        },
        {
            types: ['builtin'],
            style: { color: 'rgb(79, 193, 255)' },
        },
        {
            types: ['number', 'variable', 'inserted'],
            style: { color: 'rgb(181, 206, 168)' },
        },
        {
            types: ['operator'],
            style: { color: 'rgb(212, 212, 212)' },
        },
        {
            types: ['constant'],
            style: { color: 'rgb(100, 102, 149)' },
        },
        {
            types: ['tag', 'changed', 'function', 'keyword'],
            style: { color: 'rgb(86, 156, 214)' },
        },
        {
            types: ['attr-name'],
            style: { color: 'rgb(156, 220, 254)' },
        },
        {
            types: ['deleted', 'string', 'attr-value'],
            style: { color: 'rgb(206, 145, 120)' },
        },
        {
            types: ['class-name'],
            style: { color: 'rgb(78, 201, 176)' },
        },
        {
            types: ['char'],
            style: { color: 'rgb(209, 105, 105)' },
        },
        {
            types: ['selector'],
            style: { color: 'rgb(215, 186, 125)' },
        },
        {
            types: ['tag'],
            languages: ['markup'],
            style: { color: 'rgb(86, 156, 214)' },
        },
        {
            types: ['punctuation'],
            languages: ['markup'],
            style: { color: '#808080' },
        },
    ],
};

const vscLightTheme = {
    plain: {
        color: '#000000',
        backgroundColor: '#ffffff',
    },
    styles: [
        {
            types: ['comment'],
            style: { color: 'rgb(0, 128, 0)' },
        },
        {
            types: ['builtin'],
            style: { color: 'rgb(0, 112, 193)' },
        },
        {
            types: ['number', 'variable', 'inserted'],
            style: { color: 'rgb(9, 134, 88)' },
        },
        {
            types: ['operator'],
            style: {
                color: 'rgb(0, 0, 0)',
            },
        },
        {
            types: ['constant', 'char'],
            style: { color: 'rgb(129, 31, 63)' },
        },
        {
            types: ['tag'],
            style: { color: 'rgb(128, 0, 0)' },
        },
        {
            types: ['attr-name'],
            style: { color: 'rgb(255, 0, 0)' },
        },
        {
            types: ['deleted', 'string'],
            style: { color: 'rgb(163, 21, 21)' },
        },
        {
            types: ['changed', 'punctuation'],
            style: { color: 'rgb(4, 81, 165)' },
        },
        {
            types: ['function', 'keyword'],
            style: { color: 'rgb(0, 0, 255)' },
        },
        {
            types: ['class-name'],
            style: { color: 'rgb(38, 127, 153)' },
        },
    ],
};

const CodeHighlighter = props => {
    const { lang, value: code, darken = true, highlightLinenos, maxLines, lineHeight, collapsed, showLineNo, codesRef, onLinenoWidthChange, } = props;
    const theme = props.theme == null ? (darken ? vscDarkTheme : vscLightTheme) : props.theme;
    return (React__default["default"].createElement(Highlight__default["default"], { Prism: Highlight.Prism, code: code, theme: theme, language: lang }, ({ tokens, getLineProps, getTokenProps, className, style }) => (React__default["default"].createElement(CodeHighlighterContent, { codesRef: codesRef, highlightLinenos: highlightLinenos, lineHeight: lineHeight, maxLines: maxLines, collapsed: collapsed, showLineNo: showLineNo, tokens: tokens, getLineProps: getLineProps, getTokenProps: getTokenProps, onLinenoWidthChange: onLinenoWidthChange, className: className, style: style }))));
};
CodeHighlighter.propTypes = {
    codesRef: PropTypes__default["default"].any,
    collapsed: PropTypes__default["default"].bool,
    darken: PropTypes__default["default"].bool,
    highlightLinenos: PropTypes__default["default"].arrayOf(PropTypes__default["default"].number),
    lang: PropTypes__default["default"].string,
    lineHeight: PropTypes__default["default"].oneOfType([PropTypes__default["default"].string, PropTypes__default["default"].number]),
    maxLines: PropTypes__default["default"].number,
    onLinenoWidthChange: PropTypes__default["default"].func,
    showLineNo: PropTypes__default["default"].bool,
    theme: PropTypes__default["default"].any,
    value: PropTypes__default["default"].string.isRequired,
};
CodeHighlighter.displayName = 'YozoraCodeHighlighter';

exports.CodeHighlighter = CodeHighlighter;
exports.CodeHighlighterContent = CodeHighlighterContent;
exports["default"] = CodeHighlighter;
exports.vscDarkTheme = vscDarkTheme;
exports.vscLightTheme = vscLightTheme;
